@@ -4,14 +4,19 @@
 (in-package :nyxt)
 
 (hooks:define-hook-type keymaps-buffer (function ((list-of keymaps:keymap) buffer)
-                                                 (values &optional (list-of keymaps:keymap) buffer)))
+                                                 (values &optional (list-of keymaps:keymap) buffer))
+  "Hook to modify keymaps.
+Get a list of `nkeymaps:keymap's and `buffer' and return a new list and buffer.")
 (export-always '(hook-keymaps-buffer))
-(hooks:define-hook-type url->url (function (quri:uri) quri:uri))
+(hooks:define-hook-type url->url (function (quri:uri) quri:uri)
+  "Hook getting a `quri:uri' and returning same/another one. ")
 
 (export-always 'renderer-buffer)
 (defclass renderer-buffer ()
   ()
-  (:metaclass interface-class))
+  (:metaclass interface-class)
+  (:documentation "Renderer-specific buffer objects.
+Should be redefined by the renderer."))
 
 (defvar %default-modes '(base-mode)
   "The default modes for unspecialized buffers.
@@ -43,7 +48,26 @@ See also the `profile' slot in the `browser' class.")
    (title "")
 
    (style (theme:themed-css (theme *browser*)
+            `(:font-face :font-family "public sans" :font-style "normal" :font-weight "400" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-Regular.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "italic" :font-weight "400" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-Italic.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "normal" :font-weight "100" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-Thin.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "italic" :font-weight "100" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-ThinItalic.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "normal" :font-weight "200" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-ExtraLight.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "italic" :font-weight "200" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-ExtraLightItalic.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "normal" :font-weight "300" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-Light.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "italic" :font-weight "300" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-LightItalic.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "normal" :font-weight "500" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-Medium.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "italic" :font-weight "500" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-MediumItalic.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "normal" :font-weight "600" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-SemiBold.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "italic" :font-weight "600" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-SemiBoldItalic.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "normal" :font-weight "700" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-Bold.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "italic" :font-weight "700" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-BoldItalic.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "normal" :font-weight "800" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-ExtraBold.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "italic" :font-weight "800" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-ExtraBoldItalic.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "normal" :font-weight "900" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-Black.woff") "format('woff')")
+            `(:font-face :font-family "public sans" :font-style "italic" :font-weight "900" :src ,(format nil "url('nyxt-resource:~a')" "PublicSans-BlackItalic.woff") "format('woff')")
             `(*
+              :font-family ,theme:font-family
               :box-sizing border-box)
             `(body
               :background-color ,theme:background
@@ -52,6 +76,20 @@ See also the `profile' slot in the `browser' class.")
               :margin-top "20px")
             `(.section
               :margin-top "2em")
+            `("h1"
+              :font-size "44px")
+            `("h2"
+              :font-size "36px")
+            `("h3"
+              :font-size "32px")
+            `("h4"
+              :font-size "28px")
+            `("h5"
+              :font-size "24px")
+            `("h6"
+              :font-size "20px")
+            `("p"
+              :font-size "16px")
             `("h1,h2,h3,h4,h5,h6"
               :color ,theme:primary
               :font-family ,theme:font-family)
@@ -179,9 +217,8 @@ Useful in FFI functions where we usually specialize things against
   buffer)
 
 (export-always 'finalize-buffer)
-(defmethod finalize-buffer ((buffer buffer) &key (browser *browser*) &allow-other-keys)
-  "Finalize instantiation of BUFFER.
-Nothing to do for the simplest `buffer' type."
+(define-generic finalize-buffer ((buffer buffer) &key (browser *browser*) &allow-other-keys)
+  "Finalize instantiation of BUFFER."
   (declare (ignore browser))
   t)
 
@@ -274,10 +311,12 @@ of BUFFER."
   (unless no-hook-p
     (hooks:run-hook (buffer-after-make-hook browser) buffer)))
 
-(defmethod modes ((buffer buffer))
-  "Non-modable buffers never have modes.
-This specialization is useful to be able to call the method regardless of the
-buffer, with a meaningful result."
+(define-generic modes ((buffer buffer))
+  "Return the modes active in BUFFER.
+
+Non-`modable-buffer's never have modes.
+The default specialization on `buffer' is useful to be able to call the method
+regardless of the buffer, with a meaningful result."
   '())
 
 (defmethod modes ((buffer modable-buffer))
@@ -530,7 +569,9 @@ the buffer (which gives us more flexibility)."))
   (set-window-title))
 
 (export-always 'default-modes)
-(defgeneric default-modes (buffer)
+(define-generic default-modes (buffer)
+  "BUFFER's default modes.
+`append's all the methods applicable to BUFFER to get the full list of modes."
   (:method-combination append)
   ;; TODO: Add a warning method when passing NIL to guard the current buffer not
   ;; bound errors?
@@ -731,12 +772,27 @@ store them somewhere and `ffi-buffer-delete' them once done."))
               :opacity 0.8)
             `((:and .button (:or :visited :active))
               :color ,theme:background)
-           `("a:visited"
-              :color ,theme:secondary))))
+            `("a:visited"
+              :color ,theme:secondary)
+            `(".progress-bar-container"
+              :height "20px"
+              :width "100%")
+            `(".progress-bar-base"
+              :background-color ,theme:secondary
+              :height "100%")
+            `(".progress-bar-fill"
+              :background-color ,theme:primary
+              :height "100%"))))
   (:export-class-name-p t)
   (:export-accessor-names-p t)
   (:export-predicate-name-p t)
-  (:metaclass user-class))
+  (:metaclass user-class)
+  (:documentation "Panel buffer (also known as sidebar): small view on the side of the screen.
+
+Panels (pages openable in panel buffer with respective commands) are defined
+with `define-panel-command' and `define-panel-command-global'.
+
+Also see `panel-page'."))
 
 (define-class status-buffer (input-buffer)
   ((window
@@ -753,6 +809,14 @@ store them somewhere and `ffi-buffer-delete' them once done."))
    (display-tabs-by-last-access-p
     nil
     :documentation "Whether tabs are dynamically ordered by last access time.")
+   (glyph-left #.(alexandria:read-file-into-string
+                  (asdf:system-relative-pathname :nyxt "assets/left.svg")))
+   (glyph-right #.(alexandria:read-file-into-string
+                   (asdf:system-relative-pathname :nyxt "assets/right.svg")))
+   (glyph-reload #.(alexandria:read-file-into-string
+                    (asdf:system-relative-pathname :nyxt "assets/reload.svg")))
+   (glyph-lambda #.(alexandria:read-file-into-string
+                    (asdf:system-relative-pathname :nyxt "assets/lambda.svg")))
    (style (theme:themed-css (theme *browser*)
             `(body
               :line-height "24px"
@@ -787,14 +851,21 @@ store them somewhere and `ffi-buffer-delete' them once done."))
             `("#controls"
               :background-color ,theme:secondary
               :color ,theme:on-secondary
-              :font-weight "700"
-              :padding-left "5px"
+              :padding-left "10px"
               :overflow "hidden"
               :white-space "nowrap"
               :z-index "3"
-              :flex-basis "94px")
-            '("#execute"
-              :margin-left "10px")
+              :height "24px"
+              :flex-basis "80px")
+            `("#controls > button"
+              :width "22px"
+              :margin-left "-8px")
+            `("#reload"
+              :width "24px !important")
+            `("#controls > button:hover"
+              :background-color ,theme:accent
+              :color ,theme:on-accent
+              :opacity "1.0")
             `("#url"
               :background-color ,theme:primary
               :color ,theme:on-primary
@@ -816,7 +887,7 @@ store them somewhere and `ffi-buffer-delete' them once done."))
               :white-space "nowrap"
               :overflow-x "scroll"
               :text-align "left"
-              :padding-left "20px"
+              :padding-left "3px"
               :padding-right "20px"
               :z-index "1"
               :flex-grow "10"
@@ -934,7 +1005,14 @@ identifiers."
         (ps:chain node (set-attribute "nyxt-identifier"
                                       (ps:stringify nyxt-identifier-counter))))
       (incf nyxt-identifier-counter)
-      (dolist (child (ps:chain node children)) (add-nyxt-identifiers child))
+      (dolist (child (if (ps:chain node shadow-root)
+                         (ps:chain *array
+                                   (from (ps:@ node shadow-root children))
+                                   (concat (ps:chain *array (from (ps:@ node children)))))
+                         (ps:chain node children)))
+        (add-nyxt-identifiers child))
+      (when (ps:@ node shadow-root)
+        (ps:chain node (set-attribute "nyxt-shadow-root" "")))
       nyxt-identifier-counter)
     (setf nyxt-identifier-counter (add-nyxt-identifiers (ps:chain document body))))
   (alex:when-let ((body-json (with-current-buffer buffer
@@ -952,7 +1030,11 @@ identifiers."
   (ffi-buffer-make dead-buffer)
   dead-buffer)
 
-(defmethod document-model ((buffer buffer))
+(define-generic document-model ((buffer buffer))
+  "A wraparound accessor to BUFFER's `document-model'.
+
+In case the page changed more than `document-model-delta-threshold', runs
+`update-document-model'."
   (ps-labels :buffer buffer
     ((%count-dom-elements
       ()
@@ -1010,7 +1092,8 @@ identifiers."
    (prompter:enable-marks-p t)
    (prompter:constructor (lambda (source)
                            (mapcar #'first (nyxt::keywords (buffer source))))))
-  (:export-class-name-p t))
+  (:export-class-name-p t)
+  (:documentation "Source listing the keywords for source `buffer'."))
 
 (-> proxy-url (buffer &key (:downloads-only boolean)) *)
 (defun proxy-url (buffer &key (downloads-only nil))
@@ -1028,77 +1111,8 @@ when `proxied-downloads-p' is true."
   (and (network-buffer-p buffer)
        (eq (slot-value buffer 'status) :failed)))
 
-(export-always 'on-signal-notify-uri)
-(defmethod on-signal-notify-uri ((buffer buffer) no-url)
-  "Set BUFFER's `url' slot, then dispatch `on-signal-notify-uri' over the
-BUFFER's modes."
-  (declare (ignore no-url))
-  ;; Need to run the mode-specific actions first so that modes can modify the
-  ;; behavior of buffer.
-  (dolist (mode (modes buffer))
-    (on-signal-notify-uri mode (url buffer)))
-  (let ((view-url (ffi-buffer-url buffer)))
-    (unless (or (load-failed-p buffer)
-                (url-empty-p view-url))
-      ;; When a buffer fails to load and `ffi-buffer-url' returns an empty
-      ;; URL, we don't set (url buffer) to keep access to the old value.
-      (setf (url buffer) (ffi-buffer-url buffer))))
-  (url buffer))
-
-(export-always 'on-signal-notify-title)
-(defmethod on-signal-notify-title ((buffer buffer) no-title)
-  "Set BUFFER's `title' slot, then dispatch `on-signal-notify-title' over the
-BUFFER's modes."
-  (declare (ignore no-title))
-  (setf (title buffer) (ffi-buffer-title buffer))
-  (dolist (mode (modes buffer))
-    (on-signal-notify-title mode (url buffer)))
-  (title buffer))
-
-;; See https://webkitgtk.org/reference/webkit2gtk/stable/WebKitWebView.html#WebKitLoadEvent
-(export-always 'on-signal-load-started)
-(defmethod on-signal-load-started ((buffer buffer) url)
-  (dolist (mode (modes buffer))
-    (on-signal-load-started mode url)))
-
-(export-always 'on-signal-load-redirected)
-(defmethod on-signal-load-redirected ((buffer buffer) url)
-  (dolist (mode (modes buffer))
-    (on-signal-load-redirected mode url)))
-
-(export-always 'on-signal-load-canceled)
-(defmethod on-signal-load-canceled ((buffer buffer) url)
-  (dolist (mode (modes buffer))
-    (on-signal-load-canceled mode url)))
-
-(export-always 'on-signal-load-committed)
-(defmethod on-signal-load-committed ((buffer buffer) url)
-  (dolist (mode (modes buffer))
-    (on-signal-load-committed mode url)))
-
-(export-always 'on-signal-load-finished)
-(defmethod on-signal-load-finished ((buffer buffer) url)
-  (update-document-model :buffer buffer)
-  (dolist (mode (modes buffer))
-    (on-signal-load-finished mode url))
-  (run-thread "buffer-loaded-hook" (hooks:run-hook (buffer-loaded-hook buffer) buffer)))
-
-(export-always 'on-signal-load-failed)
-(defmethod on-signal-load-failed ((buffer buffer) url)
-  (dolist (mode (modes buffer))
-    (on-signal-load-failed mode url)))
-
-(export-always 'on-signal-button-press)
-(defmethod on-signal-button-press ((buffer buffer) button-key)
-  (dolist (mode (modes buffer))
-    (on-signal-button-press mode button-key)))
-
-(export-always 'on-signal-key-press)
-(defmethod on-signal-key-press ((buffer buffer) key)
-  (dolist (mode (modes buffer))
-    (on-signal-key-press mode key)))
-
-(hooks:define-hook-type buffer (function (buffer)))
+(hooks:define-hook-type buffer (function (buffer))
+  "Hook acting on `buffer's.")
 
 (define-command make-buffer (&rest args &key (title "") modes
                              (url (if *browser*
@@ -1226,21 +1240,28 @@ This is a low-level function.  See `buffer-delete' for the high-level version."
    :key #'id))
 
 (defun buffers-get (id)
+  "Get the `buffer' with the corresponding ID."
   (gethash id (slot-value *browser* 'buffers)))
 
 (defun buffers-set (id buffer)
+  "Set the BUFFER as the one corresponding to ID in `browser'."
   (when *browser*
     (setf (gethash id (slot-value *browser* 'buffers)) buffer)
     ;; Force setf call so that slot is seen as changed, e.g. by status buffer watcher.
     (setf (buffers *browser*) (buffers *browser*))))
 
 (defun buffers-delete (id)
+  "Remove the buffer with respective ID from the browser.
+
+Low-level function, use `buffer-delete' to properly close the buffer, or
+`delete-buffer' command from inside Nyxt."
   (remhash id (slot-value *browser* 'buffers))
   ;; Force setf call so that slot is seen as changed, e.g. by status buffer watcher.
   (setf (buffers *browser*) (buffers *browser*)))
 
 (export-always 'window-list)
 (defun window-list ()
+  "Return a list of all the open `windows'."
   (when *browser*
     (alex:hash-table-values (windows *browser*))))
 
@@ -1347,7 +1368,11 @@ proceeding."
                                         (eq buffer (current-buffer)))
                               (set-current-buffer buffer))))))
   (:export-class-name-p t)
-  (:metaclass user-class))
+  (:metaclass user-class)
+  (:documentation "Source for choosing one (or several) of the open buffers.
+
+The `prompter:actions-on-current-suggestion' are set up to preview/switch to the
+buffer currently chosen as suggestion."))
 
 (defmethod prompter:object-attributes ((buffer buffer) (source prompter:source))
   (declare (ignore source))
@@ -1533,7 +1558,9 @@ URL-DESIGNATOR is then transformed by BUFFER's `buffer-load-hook'."
    (prompter:filter-preprocessor #'prompter:filter-exact-matches)
    (prompter:actions-on-return #'buffer-load*))
   (:export-class-name-p t)
-  (:metaclass user-class))
+  (:metaclass user-class)
+  (:documentation "Source listing all the entries in history.
+Loads the entry with default `prompter:actions-on-return'."))
 
 (define-class new-url-query ()
   ((query ""
@@ -1714,7 +1741,10 @@ any.")
     (prompter::history-pushnew history (render-url url))))
 
 (export-always 'url-sources)
-(defmethod url-sources ((buffer buffer) actions-on-return)
+(define-generic url-sources ((buffer buffer) actions-on-return)
+  "Return list of `set-url' sources.
+The returned sources should have `url' or `prompter:actions-on-return' methods
+specified for their contents."
   (let ((actions-on-return (uiop:ensure-list actions-on-return)))
     (append
      (list (make-instance 'new-url-or-search-source :actions-on-return actions-on-return)
