@@ -791,3 +791,36 @@ command "
      (:li "Add cursor change on hover for buttons and links in internal pages and panel
 buffers.")
      (:li "Improve user experience of " (:nxref :command 'nyxt:pick-color) "."))))
+
+(define-version "3.5.0"
+  (:nsection :title "Features"
+    (:ul
+     (:li "Add new quick start tutorial, accesible via command "
+            (:nxref :command 'nyxt:quick-start) ".")
+     (:li "Add hinting support for pages using Shadow DOMs.")
+     (:li "Add keybinding for command " (:nxref :command 'describe-any) ".")))
+  (:nsection :title "Bug fixes"
+    (:ul
+     (:li "Fix status buffer history buttons.")
+     (:li "Fix bug in command " (:nxref :command 'nyxt/mode/repeat:repeat-key) ".")))
+  (:nsection :title "UI/UX"
+    (:ul
+     (:li "Redesign " (:a :href (nyxt-url 'new) "start page") ".")
+     (:li "Fix styling of progress bar.")
+     (:li "Fix styling of prompt buffer's input area."))))
+
+(define-version "3.6.0"
+  (:nsection :title "Features"
+    (:ul
+     (:li "Add commands for importing history from Firefox, Google Chrome,
+Chromium, Brave and Vivaldi. For instance, "
+          (:nxref :command 'nyxt/mode/history-migration:import-history-from-firefox) ".")))
+  (:nsection :title "Bug fixes"
+    (:ul
+     (:li "Fix keyscheme configuration via "
+          (:a :href (nyxt-url 'common-settings) "Common Settings Menu") ".")))
+  (:nsection :title "UI/UX"
+    (:ul
+     (:li "Add Public Sans font and set it as the default.")
+     (:li "Redesign status buffer buttons.")
+     (:li "Minor review of " (:a :href (nyxt-url 'new) "start page") "."))))
